
tic;
% Reads the csv file into a matrix
A = xlsread('wave_data_trad.xlsx');

% Set first and last month. Can be the same

% Loop over all the rows in the matrix
for i = 1:size(A,1)
Hs(i) = A(i,1);
end

% Set number of states in the markov chain
numStates = 10;

% Find upper limit for Hs values and divide the values into even bins
ul = max(Hs);
% Find state ranges - first state [0,stateRange] and so on
stateRange = ul / numStates;
% State values - stateRange, 2xstateRange and so on up til ul
stateValues = stateRange:stateRange:ul;
% Initialize 1D-matrix holding the state of each data point
HsState = zeros(length(Hs),1);

% Find each data points state
for i = 1:length(Hs)
    % For each data point
    for j = 1:numStates
        % For each state
        if Hs(i) <= stateValues(j)
            % Data point is in state j
            HsState(i) = j;
            % This data point is categorized, so we break and move to the
            % next data point
            break;
        end
    end
end

% Find transitions
transitions = zeros(numStates);
for t = 1:length(HsState)-1
    % HsState(t) represents the state and HsState(t+1) represents the state
    % it transitions to
transitions(HsState(t),HsState(t+1))=transitions(HsState(t),HsState(t+1))+1;
end

P = transitions;
% Normalize each row in the transition matrix so each row sums to 1
for i = 1:numStates
    P(i,:) = P(i,:) / sum( P(i,:) );
end

% Check to see if there are any absorbing states
% i.e. P(i,j) == 1 where i=j
absorbstate = zeros(numStates);
for i = 1:numStates
    for j = 1:numStates
        if P(i,j) == 1
            absorbstate(i,j) = absorbstate(i,j) + 1;
        end
    end
end
if sum(sum(absorbstate)) >= 1
    error('Absorbing states. Consider reducing number of states.');
end

%% Transition matrix is now ready in P

% How many state transitions to perform
% Lower this number to show how fewer replications affects results
% for example, 100, 1000, length(Hs), 10000
numReplications = 10000;

% Random number seed
rng(1235);

% Set starting state - should sample randomly
state = randi(numStates);

states = zeros(numReplications,1);

for i = 1:numReplications
	% Sample a new random value in range [0,1]
    r = rand();
    
    for j = 1:numStates
        prob = 0;
        % Accumulate probabilities
        for k = 1:j
            prob = prob + P(state,k);
        end
        
        if r <= prob
            % New state is found, j
            state = j;
            
            % Store the state we transition to
            states(i) = j;

            % Break ends the current for loop, and returns to the outer
            % loop, which will sample a new random value and start over
            break;
        end
    end
end
% If needed, Hs can be compared directly to the simulated results
simValues = zeros(numReplications,1);
for i = 1:numReplications
    simValues(i) = (states(i) * stateRange) - stateRange/2;
end
toc;