
tic;
% Reads the csv file from waveclimate.com into a matrix
D = csvread('wave_data_CV.csv',23,0);

% Set first and last month. Can be the same
firstMonth_CV = 12;
lastMonth_CV = 12;

% Loop over all the rows in the matrix
for i = 1:size(D,1)
    % Check if the month is in the range wanted
    if (D(i,2) >= firstMonth_CV) && (D(i,2) <= lastMonth_CV)
        if D(i,7) > 0 % Disregard zero and negative values
            if (exist('Hs_CV') == 0) % Hs doesn't exist
                Hs_CV = D(i,7);
            else
                % Add the value at the end of Hs
                % Sets Hs equal to Hs + the additional row which is the
                % value from A(i,7)
                Hs_CV = [ Hs_CV; D(i,7) ];
            end
        end
    end
end

% Set number of states in the markov chain
% 
% Beware of setting this too high. If there are too many states, some of
% the states will be absorbing, that is, P(j,j) = 1, which means it can
% never transition to other states.
%
%
numStates_CV = 10;

% Find upper limit for Hs values and divide the values into even bins
ul_CV = max(Hs_CV);
% Find state ranges - first state [0,stateRange] and so on
stateRange_CV = ul_CV / numStates_CV;
% State values - stateRange, 2xstateRange and so on up til ul
stateValues_CV = stateRange_CV:stateRange_CV:ul_CV;
% Initialize 1D-matrix holding the state of each data point
HsState_CV = zeros(length(Hs_CV),1);

% Find each data points state
for i = 1:length(Hs_CV)
    % For each data point
    for j = 1:numStates_CV
        % For each state
        if Hs_CV(i) <= stateValues_CV(j)
            % Data point is in state j
            HsState_CV(i) = j;
            % This data point is categorized, so we break and move to the
            % next data point
            break;
        end
    end
end

% Find transitions
transitions_CV = zeros(numStates_CV);
for t = 1:length(HsState_CV)-1
    % HsState(t) represents the state and HsState(t+1) represents the state
    % it transitions to
    transitions_CV(HsState_CV(t),HsState_CV(t+1)) = transitions_CV(HsState_CV(t),HsState_CV(t+1)) + 1;
end

G = transitions_CV;
% Normalize each row in the transition matrix so each row sums to 1
for i = 1:numStates_CV
    G(i,:) = G(i,:) / sum( G(i,:) );
end

% Check to see if there are any absorbing states
% i.e. P(i,j) == 1 where i=j
absorbstate_CV = zeros(numStates_CV);
for i = 1:numStates_CV
    for j = 1:numStates_CV
        if G(i,j) == 1
            absorbstate_CV(i,j) = absorbstate_CV(i,j) + 1;
        end
    end
end
if sum(sum(absorbstate_CV)) >= 1
    error('Absorbing states. Stopping. Consider reducing number of states or check data.');
end

%% Transition matrix is now ready in P

% How many state transitions to perform
% Lower this number to show how fewer replications affects results
% for example, 100, 1000, length(Hs), 10000
numReplications_CV = 10000;

% Random number seed
rng(1235);

% Set starting state - should sample randomly
state_CV = randi(numStates_CV);

states_CV = zeros(numReplications_CV,1);

for i = 1:numReplications_CV
	% Sample a new random value in range [0,1]
    r = rand();
    
    for j = 1:numStates_CV
        prob_CV = 0;
        % Accumulate probabilities
        for k = 1:j
            prob_CV = prob_CV + G(state_CV,k);
        end
        
        if r <= prob_CV
            % New state is found, j
            state_CV = j;
            
            % Store the state we transition to
            states_CV(i) = j;

            % Break ends the current for loop, and returns to the outer
            % loop, which will sample a new random value and start over
            break;
        end
    end
end

% If needed, Hs can be compared directly to the simulated results
simValues_CV = zeros(numReplications_CV,1);
for i = 1:numReplications_CV
    simValues_CV(i) = (states_CV(i) * stateRange_CV) - stateRange_CV/2;
end




toc;