
tic;
% Reads the csv file into a matrix
B = xlsread('wave_data_exp.xlsx');

% Set first and last month. Can be the same

% Loop over all the rows in the matrix
for i = 1:size(B,1)
Hs_exp(i) = B(i,1);
end

% Set number of states in the markov chain
% 
% Beware of setting this too high. If there are too many states, some of
% the states will be absorbing, that is, P(j,j) = 1, which means it can
% never transition to other states.
%
%
numStates_exp = 10;

% Find upper limit for Hs values and divide the values into even bins
ul_exp = max(Hs_exp);
% Find state ranges - first state [0,stateRange] and so on
stateRange_exp = ul_exp / numStates_exp;
% State values - stateRange, 2xstateRange and so on up til ul
stateValues_exp = stateRange_exp:stateRange_exp:ul_exp;
% Initialize 1D-matrix holding the state of each data point
HsState_exp = zeros(length(Hs_exp),1);

% Find each data points state
for i = 1:length(Hs_exp)
    % For each data point
    for j = 1:numStates_exp
        % For each state
        if Hs_exp(i) <= stateValues_exp(j)
            % Data point is in state j
            HsState_exp(i) = j;
            % This data point is categorized, so we break and move to the
            % next data point
            break;
        end
    end
end

% Find transitions
transitions_exp = zeros(numStates_exp);
for t = 1:length(HsState_exp)-1
    % HsState(t) represents the state and HsState(t+1) represents the state
    % it transitions to
    transitions_exp(HsState_exp(t),HsState_exp(t+1)) = transitions_exp(HsState_exp(t),HsState_exp(t+1)) + 1;
end

C = transitions_exp;
% Normalize each row in the transition matrix so each row sums to 1
for i = 1:numStates_exp
    C(i,:) = C(i,:) / sum( C(i,:) );
end

% Check to see if there are any absorbing states
% i.e. P(i,j) == 1 where i=j
absorbstate_exp = zeros(numStates_exp);
for i = 1:numStates_exp
    for j = 1:numStates_exp
        if C(i,j) == 1
            absorbstate_exp(i,j) = absorbstate_exp(i,j) + 1;
        end
    end
end
if sum(sum(absorbstate_exp)) >= 1
    error('Absorbing states. Stopping. Consider reducing number of states or check data.');
end

%% Transition matrix is now ready in P

% How many state transitions to perform
% Lower this number to show how fewer replications affects results
% for example, 100, 1000, length(Hs), 10000
numReplications_exp = 10000;

% Random number seed
rng(1235);

% Set starting state - should sample randomly
state_exp = randi(numStates_exp);

states_exp = zeros(numReplications_exp,1);

for i = 1:numReplications_exp
	% Sample a new random value in range [0,1]
    r = rand();
    
    for j = 1:numStates_exp
        prob_exp = 0;
        % Accumulate probabilities
        for k = 1:j
            prob_exp = prob_exp + C(state_exp,k);
        end
        
        if r <= prob_exp
            % New state is found, j
            state_exp = j;
            
            % Store the state we transition to
            states_exp(i) = j;

            % Break ends the current for loop, and returns to the outer
            % loop, which will sample a new random value and start over
            break;
        end
    end
end

% If needed, Hs can be compared directly to the simulated results
simValues_exp = zeros(numReplications_exp,1);
for i = 1:numReplications_exp
    simValues_exp(i) = (states_exp(i) * stateRange_exp) - stateRange_exp/2;
end



toc;